/*-------------------------------------------------------------*/
/*  File: xinitapp.c
 *  V  2.00
 *  Function to process command-line arguments, connect to the
 *  X display, and set up the main window of a generic
 *  X application.
 *
 *  X applications should include a copy of headapp.c at the biginning.
 *  If additional parameters or resources are required modify headapp.c
 *
 *  Exported Routines (appear in xwins.h): initapp
 */

#include <xwins.h>
#include <X11/Xresource.h>
#include <X11/Xatom.h>

#define  DEF_APP_GLOB    /* so that globals get defined */
#include <xwins_glob.h>

#define max(x,y)     (((x) > (y)) ? (x) : (y))

#define DEFAULT_BDWIDTH 1


char *theGeom        = NULL;

/* Parameters defined at headapp.c */
extern char *theBGcolor;
extern char *theFGcolor;
extern char *theFont;
extern char *gwFont;
extern char *gwBGcolor;
extern char *gwFGcolor;
extern char *theGeom_rsrc;
extern char *display_name;
extern char *theGeom_cline;

extern APP_PARAMS app_resources[];
extern int num_resources;

extern APP_PARAMS app_options[];
extern int num_options;
extern int options_start;

void usage();
void SetAppName();
static char *DefUseMess = "Usage: %s [-display name] [-geometry geom]\n";
static char rname[80];
static char cname[80];
/*-------------------------------------------------------------*/
/*	i n i t a p p 
 *
 * Function to start a generic X-Window application.
 * Processes the arguments.
 * If the command does not have the proper format calls  usage().
 * Sets up the global variables.
 * Reads the icon bitmap.
 * Opens the main window.
 * Sets the control variables AppDone=0, AppLock=0, FocusedWindow=theMain
 */

void initapp(argc, argv, x, y, width, height,
               app_class, icon, w_icon, h_icon, usage_mess)
int	argc;		/* Number of arguments of the command */
char	**argv;		/* Arguments values */
int	x, y;		/* Position of main window */
unsigned width, height;	/* Dimensions (pixels) of main window */
char *app_class;	/* Class of appliction (If it is NULL uses Name) */
char *icon;		/* File with the icon's bitmap */
unsigned w_icon, h_icon;/* Dimensions (pixels) of icon */
char *usage_mess;	/* Alternative usage message */
{
    int                  i, j;
    Colormap             default_cmap;
    XColor               color;
    int                  bitmask;
    XGCValues            gcv;
    XSetWindowAttributes xswa;
    char                 default_geometry[80];
    char                 rfilename[80];
    char                 *type;
    XrmValue             value;
    XrmDatabase          rdb;
    static char chess[]={0x55,0xaa,0x55,0xaa,0x55,0xaa,0x55,0xaa};
    long                 AllValuesMask;
    Pixmap               chessbitmap;
    char                 *theClass;
    char		 *usemess;

/* Initialize AppDone & AppLock */
    AppDone = 0;
    AppLock = 0;

/* No AuxPmap still */
    theAuxPmap = 0;
    theAuxPmapW = 0;
    theAuxPmapH = 0;

/* Manage options and resources */

    SetAppName(argv[0]);

    AllValuesMask = (1L<<(GCLastBit+1))-1;
    
    if(usage_mess != NULL) usemess = usage_mess;
    else usemess = DefUseMess;
    if(app_class != NULL)  theClass = app_class;
    else
    {
          theClass = malloc(strlen(theAppName)+1);
          strcpy(theClass, theAppName);
          theClass[0] = toupper(theClass[0]);
    } 
/* Parse command-line options. Since each option has a value,
 * they are processed in pairs.
 */
    for (i = options_start; i < argc; i += 2) 
    {
        for(j = 0; j < num_options; j++)
        {
            if(strcmp(app_options[j].name, argv[i]) == 0 ||
               strcmp(app_options[j].cname, argv[i]) == 0)
            {
                *app_options[j].p_value_string = argv[i+1];
                break;
            }
        }
        if(j >= num_options)
            usage(usemess);
    }

/*  Open connection to display selected by user */

    if ((theDisplay = XOpenDisplay(display_name)) == NULL) 
    {
        fprintf(stderr, "%s: can't open display named %s\n", 
                argv[0], XDisplayName(display_name));
        exit(1);
    }

/* Get resources from the resource file */
    XrmInitialize();
    strcpy(rfilename, getenv("HOME"));
    strcat(rfilename, "/.Xdefaults");
    rdb = XrmGetFileDatabase(rfilename);
    if(rdb != NULL) 
    {
        for (i = 0; i < num_resources; i++)
        {
/* Construct a complete resource name by appending the
 * resource name to the application's name.
 */
            strcpy(rname, theAppName);
            strcat(rname, "*");
            strcat(rname, app_resources[i].name);
            strcpy(cname, theClass);
            strcat(cname, "*");
            strcat(cname, app_resources[i].cname);

            if(XrmGetResource(rdb, rname, cname,
                              &type, &value))
            {
                *app_resources[i].p_value_string = value.addr;
            }
        }
    }

/* Set up colors and fonts */
/* Font of main */
    if ((theFontStruct = XLoadQueryFont(theDisplay, 
                                           theFont)) == NULL) 
    {
        fprintf(stderr, "%s: display %s cannot load font %s\n",
                theAppName, DisplayString(theDisplay), theFont);
        exit(1);
    }
    
/* gw font */
    if(gwFont == NULL) gwFontStruct = theFontStruct;
     else
     {
          if ((gwFontStruct = XLoadQueryFont(theDisplay,
                                           gwFont)) == NULL)
          {
		fprintf(stderr, "%s: display %s cannot load font %s\n",
			theAppName, DisplayString(theDisplay), gwFont);
		exit(1);
	  }


     }
/* Now select the colors using the default colormap */

    default_cmap = DefaultColormap(theDisplay, 
                                   DefaultScreen(theDisplay));

/* Main window's background color */
    if (XParseColor(theDisplay, default_cmap, theBGcolor, 
                    &color) == 0 ||
        XAllocColor(theDisplay, default_cmap, &color) == 0)

/* Use white background in case of failure */
        theBGpix = WhitePixel(theDisplay, 
                              DefaultScreen(theDisplay));
    else
        theBGpix = color.pixel;

/* Main window's foreground color */
    if (XParseColor(theDisplay, default_cmap, theFGcolor, 
                    &color) == 0 ||
        XAllocColor(theDisplay, default_cmap, &color) == 0)
/* Use black foreground in case of failure */
        theFGpix = BlackPixel(theDisplay, 
                              DefaultScreen(theDisplay));
    else
        theFGpix = color.pixel;

/* BG of gw */
    if(gwBGcolor == NULL) gwBGpix = theBGpix;
    else
    {
	if (XParseColor(theDisplay, default_cmap, gwBGcolor, &color) == 0 ||
		XAllocColor(theDisplay, default_cmap, &color) == 0)

		gwBGpix = theBGpix; 
	else
		gwBGpix = color.pixel;
    }
/* FG of gw */
    if(gwFGcolor == NULL) gwFGpix = theFGpix;
    else
    {
	if (XParseColor(theDisplay, default_cmap, gwFGcolor, &color) == 0 ||
		XAllocColor(theDisplay, default_cmap, &color) == 0)

		gwFGpix = theFGpix; 
	else
		gwFGpix = color.pixel;
    }


/* Fill out a XsizeHints structure to inform the window manager
 * of desired size and location of main window.
 */
    if((p_XSH = XAllocSizeHints()) == NULL)
    {
        fprintf(stderr, "Error allocating size hints!\n");
        exit(1);
    }
    p_XSH->flags = (PPosition | PSize | PMinSize);
    p_XSH->height = height;
    p_XSH->min_height = p_XSH->height;
    p_XSH->width = width;
    p_XSH->min_width = p_XSH->width;
    p_XSH->x = x;
    p_XSH->y = y;

/* Construct a default geometry string */
    sprintf(default_geometry, "%dx%d+%d+%d", p_XSH->width,
            p_XSH->height, p_XSH->x, p_XSH->y);
    theGeom = default_geometry;

/* Override the geometry, if necessary */
    if(theGeom_cline != NULL) theGeom = theGeom_cline;
    else
        if(theGeom_rsrc != NULL) theGeom = theGeom_rsrc;

/* Process the geometry specification */
    bitmask =  XGeometry(theDisplay, DefaultScreen(theDisplay), 
                   theGeom, default_geometry, DEFAULT_BDWIDTH, 
                   1, 1, 0, 0, &(p_XSH->x), &(p_XSH->y),
                   &(p_XSH->width), &(p_XSH->height));
        
/* Check bitmask and set flags in XSizeHints structure */
    if (bitmask & (XValue | YValue)) p_XSH->flags |= USPosition;
    if (bitmask & (WidthValue | HeightValue))
                                     p_XSH->flags |= USSize;

/* Create the main window using the position and size 
 * information derived above. For border color, use the 
 * foreground color.
 */
    theMain = XCreateSimpleWindow(theDisplay, 
                    DefaultRootWindow(theDisplay), 
                    p_XSH->x, p_XSH->y, p_XSH->width, p_XSH->height, 
                    DEFAULT_BDWIDTH, theFGpix, theBGpix);

/* Set up class hint */
    if((p_CH = XAllocClassHint()) == NULL)
    {
        fprintf(stderr, "Error allocating class hint!\n");
        exit(1);
    }
    p_CH->res_name = theAppName;
    p_CH->res_class = theClass;

/* Set up XTextProperty for window name and icon name */
    if(XStringListToTextProperty(&theAppName, 1, &WName) == 0)
    {
        fprintf(stderr, "Error creating XTextProperty!\n");
        exit(1);
    }
    if(XStringListToTextProperty(&theAppName, 1, &IName) == 0)
    {
        fprintf(stderr, "Error creating XTextProperty!\n");
        exit(1);
    }

/* Set up W M hints */
    if((p_XWMH = XAllocWMHints()) == NULL)
    {
        fprintf(stderr, "Error allocating Window Manager hints!\n");
        exit(1);
    }
    p_XWMH->flags = (InputHint|StateHint);
    if(icon != NULL)
    {
          p_XWMH->icon_pixmap = XCreatePixmapFromBitmapData(
			      theDisplay, theMain,
                              icon, w_icon, h_icon,
			      theFGpix, theBGpix, 1);
         if(p_XWMH->icon_pixmap == None)
               fprintf(stderr, "Failed to create pixmap\n");
         else
         {
               p_XWMH->flags = p_XWMH->flags | IconPixmapHint ;
         }
    }
    p_XWMH->input = True;
    p_XWMH->initial_state = NormalState;

    XSetWMProperties(theDisplay, theMain, &WName, &IName, argv, argc,
                     p_XSH, p_XWMH, p_CH);

/* Create a graphics context for the main window */    

    gcv.font = theFontStruct->fid;
    gcv.foreground = theFGpix;
    gcv.background = theBGpix;
    theGC = XCreateGC(theDisplay, theMain, 
                (GCFont | GCForeground | GCBackground), &gcv);

/* Create other graphics contexts */

    InvertGC = XCreateGC(theDisplay, theMain, (long)0,NULL);
    XCopyGC(theDisplay, theGC, AllValuesMask, InvertGC);
    XSetPlaneMask(theDisplay, InvertGC, theFGpix^theBGpix);
    XSetFunction(theDisplay, InvertGC,GXinvert);
 
    chessbitmap = XCreateBitmapFromData(theDisplay,
                  DefaultRootWindow(theDisplay), chess, 8,8);

    FillGC = XCreateGC(theDisplay, theMain, (long)0,NULL);
    XCopyGC(theDisplay, theGC, AllValuesMask, FillGC);
    XSetFillStyle(theDisplay, FillGC, FillOpaqueStippled);
    XSetStipple(theDisplay, FillGC, chessbitmap);
 
    LightGC = XCreateGC(theDisplay, theMain, (long)0,NULL);
    XCopyGC(theDisplay, theGC, AllValuesMask, LightGC);
    XSetForeground(theDisplay,  LightGC, theBGpix);
    XSetFillStyle(theDisplay, LightGC, FillStippled);
    XSetStipple(theDisplay, LightGC, chessbitmap);
 
/* Create a graphics context for the graphic window */    

    gcv.font = gwFontStruct->fid;
    gcv.foreground = gwFGpix;
    gcv.background = gwBGpix;
    gwGC = XCreateGC(theDisplay, theMain, 
                (GCFont | GCForeground | GCBackground), &gcv);

/* Set main window's attributes (colormap, bit_gravity) */

    xswa.colormap = DefaultColormap(theDisplay, 
                                    DefaultScreen(theDisplay));
    xswa.bit_gravity = NorthWestGravity;
    XChangeWindowAttributes(theDisplay, theMain, (CWColormap | 
                            CWBitGravity), &xswa);

/* Create atoms for WM communication */

    ATOM_WM_PROTOCOLS = XInternAtom(theDisplay, "WM_PROTOCOLS", False);
    ATOM_WM_DELETE_WINDOW = XInternAtom(theDisplay,
                             "WM_DELETE_WINDOW", False);

/* Include WM_DELETE_WINDOW in WM_PROTOCOLS Property */

    XChangeProperty(theDisplay, theMain, ATOM_WM_PROTOCOLS, XA_ATOM, 32,
                    PropModeReplace, 
		    (unsigned char*)&ATOM_WM_DELETE_WINDOW, 1);

/* Select Exposure events for the main window */

    XSelectInput(theDisplay, theMain,
           ExposureMask|StructureNotifyMask|FocusChangeMask);

/* Initialize FocusedWindow */
    FocusedWindow = theMain;

/* Map the main window */

    XMapWindow(theDisplay, theMain);
    XFlush(theDisplay);
    if(XGetWindowAttributes(theDisplay, theMain, &MainXWA) == 0)
    {
        fprintf(stderr, "Error getting attributes of Main.\n");
        exit(2);
    }
}

/*-----------------------------------------------------------------------*/
void usage (message)
char *message;
{
    fprintf (stderr, message, theAppName);
    exit (1); 
}
/*-----------------------------------------------------------------------*/
void SetAppName(fullpath)
/* Strips the directory from the Application Name */
char fullpath[];
{
     int i,j;
		i=0; j=0;

		while (fullpath[i] != '\0')
		{
			if(fullpath[i] == '/')j=i+1;
			i=i+1;
		}

		theAppName = &fullpath[j];
}
