/*   File: xinline.c
 *  V 1.06
 *  Implememts a window for input of a line of text.
 *
 *  Exported Routines (appear in xwins.h):
 *	MakeXInputLine, DestroyXInputLine, RefreshInputLine,
 *	StartInputLine, HideCursor. 
 */
/*-------------------------------------------------------------------*/
#include <X11/keysym.h>
/* ooblique may have this other name */
#ifndef XK_ooblique
#define XK_ooblique XK_oslash
#endif
#ifndef XK_NoSymbol
#define XK_NoSymbol XK_VoidSymbol
#endif
/* If dead_grave is not defined, no dead key is defined */
#ifndef XK_dead_grave
#define XK_dead_grave		XK_NoSymbol
#define XK_dead_acute		XK_NoSymbol
#define XK_dead_circumflex	XK_NoSymbol
#define XK_dead_tilde		XK_NoSymbol
#define XK_dead_diaeresis	XK_NoSymbol
#define XK_dead_abovering	XK_NoSymbol
#define XK_dead_cedilla		XK_NoSymbol
#define XK_dead_macron		XK_NoSymbol
#define XK_dead_breve		XK_NoSymbol
#define XK_dead_abovedot	XK_NoSymbol
#define XK_dead_doubleacute	XK_NoSymbol
#define XK_dead_caron		XK_NoSymbol
#define XK_dead_ogonek		XK_NoSymbol
#endif
/* Some times dead_abovering is called dead_degree */
#ifndef XK_dead_abovering
#define XK_dead_abovering XK_dead_degree
#endif
#include <xwins.h>
#include <xwins_glob.h>

typedef struct{
int       x_off,
          y_off,
          letter_w,
          letter_h,
          letter_y,
          cursor_p,
          n_max,
          last,
          first,
          buf_size;
char      *text;
void      (*return_action)();
void      *data;
int       comp_status;    /* For compose characters */
KeySym    PrevKS;         /* Previous Character for compose */
}  D_INPUT_LINE;

/*---------------------------------------------------------------------*/
/*   M a k e X I n p u t L i n e
 *
 *   Makes a window for input of text
 */
XWIN *MakeXInputLine(x, y, nmax, x_off, y_off, y_gap, p_w, p_h, bdwidth,
                bdcolor, bgcolor, parent, text, text_size,
                return_action, action_data)

int     x, y;           /* Position of upper left corner */
int     nmax;           /* Maximum number of letters displayed */
int     x_off, y_off;   /* X and Y offsets */
int     y_gap;          /* Space below the text */
unsigned  *p_w, *p_h;   /* Width and height of window */
unsigned   bdwidth;     /* Width of border */
unsigned long bdcolor;  /* Color of border */
unsigned long bgcolor;  /* Color of Background */
Window  parent;         /* Parent window */
char    *text;          /* text buffer */
int     text_size;      /* size of buffer */
void    (*return_action)();  /* Action on RETURN key */
void    *action_data;   /* Data for return_action function */

{
        D_INPUT_LINE    *d;
        XWIN            *new_xwin;
        void            inputline_handler();

 
/* Allocate a XWIN structure */
    if((new_xwin = (XWIN *)calloc(1, sizeof(XWIN))) == NULL)
    {
        fprintf(stderr, "No memory for window\n");
        exit(1);
    }
 
/* Allocate a D_INPUT_LINE structure */
    if((d = (D_INPUT_LINE *)calloc(1, sizeof(D_INPUT_LINE))) == NULL)
    {
        fprintf(stderr, "No memory for window data\n");
        exit(1);
    }
        new_xwin->parent = parent;
        new_xwin->event_handler = inputline_handler;
        new_xwin->data  = d;
        d->x_off = x_off;
        d->y_off = y_off;
        d->n_max = nmax - 1;
        d->buf_size = text_size;
        d->text = text;
        d->return_action = return_action;
        d->data = action_data;
        d->letter_w = theFontStruct->max_bounds.width;
        d->letter_h = theFontStruct->max_bounds.ascent +
                                theFontStruct->max_bounds.descent;
        d->letter_y = theFontStruct->max_bounds.ascent + y_off;
        *p_w = 2*x_off + d->letter_w*nmax;
        *p_h = y_off + d->letter_h + y_gap;
        d->last = strlen(text)-1;
        if(d->last > d->n_max)
        {
                d->first = d->last - d->n_max;
                d->cursor_p = nmax;
        }
        else
        {
                d->first = 0;
                d->cursor_p = d->last + 1;
        }

        new_xwin->xid = XCreateSimpleWindow(theDisplay, parent, x, y,
                                *p_w, *p_h, bdwidth, bdcolor,
                                bgcolor);

/* Save new_xwin as data associated with this window id
 * and the context "xwin_context"
 */
    if(XSaveContext(theDisplay, new_xwin->xid, xwin_context,
                 (caddr_t) new_xwin) != 0)
    {
        fprintf(stderr, "Error saving xwin_context data");
        exit(1);
    }
 
    XSelectInput(theDisplay, new_xwin->xid,
        ExposureMask | ButtonPressMask | KeyPressMask);
 
    XMapWindow(theDisplay, new_xwin->xid);
    return (new_xwin);

}

/*--------------------------------------------------------------------*/
/*     D e s t r o y X I n p u t L i n e
 *
 *   Destroys an XInputLine
 */
void DestroyXInputLine(p_xwin)
XWIN *p_xwin;			/* Pointer to XWIN of XInputLine */
{       D_INPUT_LINE *p_data;
        p_data = (D_INPUT_LINE*)(p_xwin->data);
        XDestroyWindow(theDisplay, p_xwin->xid);
	XDeleteContext(theDisplay, p_xwin->xid, xwin_context);
        free(p_data);
	if(FocusedWindow == p_xwin->xid)FocusedWindow = theMain;
        free(p_xwin);
}
/*---------------------------------------------------------------------*/
/*        W r i t e C h a r
 *
 *   Writes a character in position n of an input line
 */
int WriteChar(p_inline, n, c)
XWIN *p_inline;
int  n;
char c;
{
     D_INPUT_LINE   *d;
     char           letter;
     int x;

     d = (D_INPUT_LINE *) p_inline->data;
     x = d->x_off + d->letter_w * n;
     letter = c;
     XClearArea(theDisplay, p_inline->xid, x, d->y_off,
                         d->letter_w, d->letter_h, False);
     if( n>= 0 && n<= d->n_max)
     {
          XDrawString(theDisplay, p_inline->xid, theGC,
                         x, d->letter_y, &letter, 1);
          return 1;
     }
     else
          return 0;
}
/*---------------------------------------------------------------------*/
/*        M o v e C u r s o r
 *
 *  Moves cursor to  position n
 */
int   MoveCursor(p_inline, n)
XWIN  *p_inline;
int   n;
{
     D_INPUT_LINE *d;
     int  x; 
     void   HideCursor();

     d = (D_INPUT_LINE *) p_inline->data;

     HideCursor(p_inline);
     if( n>= 0 && n <= d->n_max+1)
     {
          x = d->x_off + d->letter_w*n;

          XFillRectangle (theDisplay, p_inline->xid, InvertGC,
                    x, d->y_off, d->letter_w, d->letter_h);
          d->cursor_p = n;
          return 1;
     }
     else
          return 0;
}
/*---------------------------------------------------------------------*/
/*   H i d e C u r s o r
 *
 *   Erases the cursor
 */
void  HideCursor(p_inline)
XWIN  *p_inline;			/* Pointer to XWIN of XInputLine */
{
     D_INPUT_LINE *d;
     char	  letter;

     d = (D_INPUT_LINE *)p_inline->data;
     if(d->cursor_p > d->last - d->first)
	letter = ' ';
     else
        letter = d->text[d->first + d->cursor_p];
     WriteChar(p_inline, d->cursor_p, letter);
}
/*---------------------------------------------------------------------*/
/*   R e f r e s h I n p u t L i n e
 *
 *   Refreshes an input line
 */
void  RefreshInputLine(p_inline)
XWIN  *p_inline;			/* Pointer to XWIN of XInputLine */
{
     D_INPUT_LINE *d;
     int i, imax;
     char *text;

     d = (D_INPUT_LINE *)p_inline->data;
     text = d->text;

     XClearWindow(theDisplay, p_inline->xid);

     imax = d->last - d->first;
     if(imax > d->n_max) imax = d->n_max;

     for( i=0; i<=imax; i++)
               WriteChar(p_inline,i,text[i+d->first]);

     if(FocusedWindow == p_inline->xid) MoveCursor(p_inline, d->cursor_p);
}
/*--------------------------------------------------------------------*/
/*    I n p u t L i n e D e l e t e
 *
 *  Deletes the character preceding the cursor.
 */
int InputLineDelete(p_inline)
XWIN *p_inline;
{
     D_INPUT_LINE *d;
     int i, imax;
     char *text;

     d = (D_INPUT_LINE *)p_inline->data;
     text = d->text;

     imax = d->last - d->first;
     if(imax > d->n_max)imax = d->n_max;

     if(d->cursor_p == 0) return 0;
     else if((d->last - d->first == d->n_max) && (d->first > 0))
     {
/* The left side shifts to the right */
          for( i = d->cursor_p + d->first -1; i<=d->last; i++)
               text[i] = text[i+1];
          d->first = d->first - 1;
          d->last = d->last - 1;
          for(i=0; i< d->cursor_p; i++)
               WriteChar(p_inline, i, text[i + d->first]);
     }
     else
     {
/* The right side shifts to the left */
          for(i=d->cursor_p-1; i<imax; i++)
          {
               text[i + d->first] = text[i + d->first + 1];
               WriteChar(p_inline, i, text[i+d->first]);
          }
/* copies including null character */
          for(i=imax+d->first; i<=d->last; i++)
                    text[i] = text[i+1];
          if(d->last-d->first > d->n_max)
               WriteChar(p_inline, imax, text[imax+d->first]);
          else
               WriteChar(p_inline, imax, ' ');
          d->last = d->last - 1;
	  MoveCursor(p_inline,d->cursor_p-1);
     }
     return 1;
}
/*---------------------------------------------------------------------*/
/*   L e f t A r r o w
 *
 *  Action of the left arrow
 */
void LeftArrow(p_inline)
XWIN *p_inline;
{
     D_INPUT_LINE *d;

     d = (D_INPUT_LINE *)p_inline->data;
     if( d->cursor_p > 0)
          MoveCursor(p_inline, d->cursor_p - 1);
     else if(d->first > 0)
     {
          d->first = d->first - 1;
          RefreshInputLine(p_inline);
     }
}
/*---------------------------------------------------------------------*/
/*   R i g h t A r r o w
 *
 *  Action of the right arrow
 */
void RightArrow(p_inline)
XWIN *p_inline;
{
     D_INPUT_LINE *d;

     d = (D_INPUT_LINE *)p_inline->data;
     if( d->cursor_p <= d->n_max && d->cursor_p <= d->last - d->first )
          MoveCursor(p_inline, d->cursor_p + 1);
     else if( d->last - d->first > d->n_max)
     {
          d->first = d->first + 1;
          RefreshInputLine(p_inline);
     }
}
/*-------------------------------------------------------------*/
/*   P u t C h a r
 *
 *   Inserts a character in the input line.
 */
int  PutChar(p_inline,c)
XWIN *p_inline;
char c;
{
     D_INPUT_LINE   *d;
     char           *text;
     int            i, imax;

     d = (D_INPUT_LINE *)p_inline->data;
     text = d->text;
     if(d->last < d->buf_size -2)
     {
          imax = d->last - d->first;
          if( imax < d->n_max)
          {
               if(d->cursor_p>imax)
               {
                    MoveCursor(p_inline,imax+2);
                    d->last = d->last + 1;
                    text[d->last] = c;
		    text[d->last + 1] = '\0';
                    WriteChar(p_inline,imax+1,c);
               }
               else
               {
                    for( i=d->last + 1; i>=d->cursor_p + d->first; i--)
                         text[i+1] = text[i];
                    text[d->cursor_p + d->first] = c;
                    for(i=d->cursor_p; i<= imax+1; i++)
                         WriteChar(p_inline, i, text[i+d->first]);
                    MoveCursor(p_inline,d->cursor_p + 1);
                    d->last = d->last + 1;
               }
          }
          else
          {
               for( i=d->last + 1; i>=d->cursor_p + d->first; i--)
                         text[i+1] = text[i];
               text[d->cursor_p + d->first] = c;
               d->last = d->last +1;
               if(d->cursor_p <= d->n_max)
                    d->cursor_p = d->cursor_p +1;
               else
                    d->first = d->first + 1;
               RefreshInputLine(p_inline);
          }
          return 1;
     } 
     else           /* Buffer is full */
          return 0;
}
/*---------------------------------------------------------------------*/
/*   S t a r t I n p u t L i n e
 *
 *   Sets the cursor, gives focus and refreshes an input line.
 */
void  StartInputLine(p_inline)
XWIN  *p_inline;
{
     D_INPUT_LINE *d;

     d = (D_INPUT_LINE *)p_inline->data;
     d->last = strlen(d->text) - 1;
     if(d->last > d->n_max)
     {
          d->first = d->last - d->n_max;
          d->cursor_p = d->n_max;
     }
     else
     {
          d->cursor_p = d->last + 1;
          d->first = 0;
     }
     d->comp_status = 0;
     d->PrevKS = NoSymbol;
     FocusedWindow = p_inline->xid;
     RefreshInputLine(p_inline);
     XSetInputFocus(theDisplay, p_inline->xid, RevertToNone, CurrentTime);
}
/*---------------------------------------------------------------------*/
int SearchTable(ks,tabl,ini,fin)
KeySym ks;
KeySym *tabl;
int ini,fin;
{
	int i,out;

	out = -1;
	for(i=ini;i<=fin;i++)
		if(ks == tabl[i]) out=i;
	return out;
}
/*---------------------------------------------------------------------*/
void inputline_handler(p_inline)
XWIN *p_inline;
{
#include "xwinsCompose.h"

        D_INPUT_LINE *d;
        int     i, n, ind, ini, fin, continua, swcompose;
        char    buffer[10],letter;
        int     buffersize =10;
        KeySym  ks;

        d = (D_INPUT_LINE*) p_inline->data;

        switch(theEvent.type)
        {
           case Expose:
                RefreshInputLine(p_inline);
                break;
           case ButtonPress:
		if(FocusedWindow != p_inline->xid)
		{ Window wp;
		  XEvent event;
			wp = FocusedWindow;
			event.type = Expose;
			event.xany.window = wp;
                	FocusedWindow = p_inline->xid;
			XSetInputFocus(theDisplay, p_inline->xid,
					 RevertToNone, CurrentTime);
			XSendEvent(theDisplay, wp, False, ExposureMask, &event);
		}
                n = 0;
                for (i=0; i <= d->n_max; i++)
                   if(theEvent.xbutton.x >
			 d->x_off + d->letter_w * i + d->letter_w/2)
                                n = i+1;
		if(n > d->last - d->first + 1) n = d->last - d->first + 1;
                MoveCursor(p_inline, n);
                break;
           case KeyPress:
	    if(FocusedWindow == p_inline->xid)
	    {
                n = XLookupString((XKeyEvent*)&theEvent,
					 buffer, buffersize, &ks, NULL);
		if( n > 1 )
		{
			XBell(theDisplay,100);
			fprintf(stderr,"Key binding is not supported\n");
			break;
		}
		continua = 0;
		switch(ks)
		{
		   case XK_NoSymbol:
			break;
		   case XK_Right:
			d->comp_status = 0;
			RightArrow(p_inline);
                       	break;
		   case XK_Left:
			d->comp_status = 0;
			LeftArrow(p_inline);
			break;
		   case XK_Multi_key:
			if(d->comp_status == 0)
				d->comp_status=1;
			else
			{
				d->comp_status=0;
				XBell(theDisplay,100);
			}
			break;

		   default:
			continua = 1;
		}

		if( IsModifierKey(ks)|| ks == XK_Mode_switch || continua == 0)
							 break;
		if(ks == XK_Delete || ks == XK_BackSpace)
                {
			d->comp_status = 0;
                        InputLineDelete(p_inline);
		}else if( ks == XK_Return)
		{
			d->comp_status = 0;
                        if(d->return_action != NULL)
                                (*d->return_action)(d->data);
		}else
		{
		 if( ks == XK_dead_grave ||
                   ks == XK_dead_acute||
                   ks == XK_dead_circumflex||
                   ks == XK_dead_diaeresis||
                   ks == XK_dead_tilde||
                   ks == XK_dead_cedilla||
                   ks == XK_dead_abovering||
                   ks == XK_dead_macron||
                   ks == XK_dead_breve||
                   ks == XK_dead_abovedot||
                   ks == XK_dead_doubleacute||
                   ks == XK_dead_caron||
                   ks == XK_dead_ogonek
/* These are not Latin and are frequently not defined: 
                   || ks == XK_dead_iota||
                   ks == XK_dead_voiced_sound||
                   ks == XK_dead_semivoiced_sound||
                   ks == XK_dead_belowdot||
                   ks == XK_dead_hook||
                   ks == XK_dead_horn
  */
			)
			if(d->comp_status == 0)
		  	{
				d->PrevKS = ks;
				d->comp_status = 3;
				break;
			}
		  swcompose = d->comp_status;
		  switch(swcompose)
		  {
		   case 0:  /* Normal character */
		 	/* this works for Latin-1 */
			letter = (char)ks;
			if( !PutChar(p_inline, letter) )
			{
			   XBell(theDisplay,100);
			   fprintf(stderr," Buffer of text is full\n");
			}
			break;
		   case 1: /* First letter of compose */
			if(n == 0)
			{ /* XLookupString handles compose characters */
				d->comp_status = 0;
				break;
			}
			d->PrevKS = ks;
			d->comp_status = 2;
			break;
		   case 2: /* Second letter of compose */
			ind = SearchTable(d->PrevKS,TabC1,0,NCK1) ;
			if(ind < 0)  /* First letter is not in table */
			{
				d->comp_status = 0;
				XBell(theDisplay,100);
				break;
			} 
			if( ind == 0) ini = 0;
			else ini = indTC[ind -1] + 1;
			fin = indTC[ind];
			ind = SearchTable(ks,TabC2,ini,fin) ;
	                if(ind < 0) /* Second letter is not in table */
                        {
       	                        d->comp_status = 0;
				XBell(theDisplay,100);
				break;
                         }
		 	/* this works for Latin-1 */
	  		 letter = (char)resultC[ind];
			 if( !PutChar(p_inline, letter) )
			 {
			    XBell(theDisplay,100);
			    fprintf(stderr," Buffer of text is full\n");
			 }
			 d->comp_status = 0;
                         break;
		    case 3: /* Accented letter */
			 ind = SearchTable(d->PrevKS,TabDA1,0,NDAK1);
			 if(ind < 0) /*  Dead accent is not in table */
			 {
				d->comp_status = 0;
				XBell(theDisplay,100);
				break;
			 }
                         if( ind == 0) ini = 0;
                         else ini = indTDA[ind -1] + 1;
			 fin = indTDA[ind];
			 ind = SearchTable(ks,TabDA2,ini,fin);
			 if(ind < 0) /* Letter is not in table */
			 {
				d->comp_status = 0;
				XBell(theDisplay,100);
				break;
			  }
		 	/* this works for Latin-1 */
			 letter = (char)resultDA[ind];
		 	 if( !PutChar(p_inline, letter) )
			 {
			    XBell(theDisplay,100);
			    fprintf(stderr," Buffer of text is full\n");
			 }
			 d->comp_status = 0;
                         break;
		  } /* End case of status */
		 } /* End if ks */
	    } /* End if Focuosed */
	    break;
        } /* End case of theEvent.type */
} /* End handler */
/*---------------------------------------------------------------------*/
