/*  File: xinpanel.c
 *  V 1.06
 *   Implememnts a panel for input of text data. 
 *
 *   Exported Routines: MakeXInputPanel, DestroyXInputPanel.
 */ 
#include <stdio.h> 
#include <stdarg.h>
#include <string.h>
 
#include <xwins.h>
#include <xwins_glob.h>

typedef void    (*P_FUNC)();
typedef char   *P_CHAR;

typedef struct D_ITEM { 
int     y_label;
int     x_line;
char    *label; 
char    *text;
char    buff[80];
XWIN    *InputLine;
XWIN    *panel;
struct D_ITEM  *next;
} D_ITEM;


typedef   struct{
char      *title;
int       xt;
int       yt;
int       x_label;
int       dy;
int       l_line;
XWIN      *bok, *bref, *bcan;
P_FUNC     ok_action, cancel_action;
caddr_t    ok_data, cancel_data;
D_ITEM     *list;
}D_INPUT_PANEL;

void    InputPanel_handler();
void    cancel_button_action();
void    ok_button_action();
void    refresh_button_action();
void    cr_action();
void	DestroyXInputPanel();

#define MARGIN 10
#define LINE_LENGTH 30
#define LINE_GAP 3
#define X_OFF 5
#define Y_OFF 5
#define Y_GAP 3
#define BUTTON_BD 3
#define BUTTON_BD_DEF 6
/*---------------------------------------------------------------------*/
/*     M a k e X I n p u t P a n e l
 *
 *  To produce a panel for input of text data.
 *  There are n input lines identified by a label.
 *  There are also OK, CANCEL and REFRESH buttons.
 *  The data is accepted pressing the OK button.
 *  By pressing the REFRESH button you can start again
 *  the input of data.
 *  When n>1 CR moves the cursor to the next line.
 *  When n = 1 CR accepts the data.
 *
 *  Calling stament:
 *  p_xwin = MakeXInputPanel(unsigned long bdcolor, unsigned long bgcolor,
 *                    Window parent, unsigned parent_w, unsigned parent_h,
 *                    char *title,
 *                    void (*ok_action)(), caddr_t ok_data,
 *                    void (*cancel_action)(), caddr_t cancel_data,
 *                    char *label1, char *text1,
 *                    ...,
 *                    NULL);
 */

XWIN      *MakeXInputPanel(             /* Old-style definition not allowed */
     unsigned long bdcolor,		/* Color of border */
     unsigned long bgcolor,		/* Color of background */
     Window         parent,		/* XID of parent window */
     unsigned int parent_w,		/* Width of parent window */
     unsigned int parent_h, 		/* Height of parent window */
     char*        title,		/* Title of InputPanel */
     P_FUNC       ok_action,		/* Action of OK button */
     caddr_t      ok_data,		/* argument of ok_action */
     P_FUNC       cancel_action,	/* Action of Cancel button */
     caddr_t      cancel_data,		/* argument of cancel_action */
                   ... )
{
     va_list        argp;		/* To point to the variable
						argument list */
     XWIN           *new_panel;
     D_ITEM         *p_i, *p_i_old;
     D_INPUT_PANEL  *d_panel;
     int            inline_num;
     int            lmax;
     char           *label;
     int            window_width, window_height;
     int            window_x, window_y;
     int            title_length;
     int            row_width;
     int            inline_height;
     int            y_inline;
     int            x_butt;
     unsigned       w,h;
     char           *cancel_text = "Cancel";
     char           *ok_text = "OK";
     char           *refresh_text = "Refresh";
     int            l_cancel, l_ok, l_refresh;
     unsigned       ok_bd;
     P_FUNC         line_action;
     caddr_t        line_data;

     va_start(argp,cancel_data);

/* Allocate memory for panel data */
     d_panel = (D_INPUT_PANEL*)calloc(1,sizeof(D_INPUT_PANEL));
     if(d_panel ==NULL)
     {
          fprintf(stderr, "No memory for panel data\n");
          exit(1);
     }

/* Allocate a XWIN structure */
     if((new_panel = (XWIN*)calloc(1,sizeof(XWIN))) == NULL)
     {
          fprintf(stderr, "No memory for window\n");
          exit(1);
     }
     new_panel->data = d_panel;
     new_panel->event_handler = InputPanel_handler;
     d_panel->title = title;
     d_panel->ok_action = ok_action;
     d_panel->ok_data = ok_data;
     d_panel->cancel_action = cancel_action;
     d_panel->cancel_data = cancel_data;

     lmax =0;
     inline_num = 0;
     d_panel->list = NULL;
     p_i_old = NULL;

/* Get rest of the items one by one */
     while((label = va_arg(argp, P_CHAR)) != NULL)
     {
          if((p_i = (D_ITEM*)calloc(1,sizeof(D_ITEM))) == NULL)
          {
               fprintf(stderr, "No memory for panel items\n");
               exit(1);
          }
          if(d_panel->list == NULL)
          {
               d_panel->list = p_i;
          }
          if(p_i_old != NULL)
          {
               p_i_old->next = p_i;
          }
          inline_num++;
          p_i->label = label;
          p_i->text  = va_arg(argp, P_CHAR);
          p_i->x_line = strlen(label);
          if(p_i->x_line>lmax) lmax = p_i->x_line;
          p_i->panel = new_panel;
          p_i_old = p_i;
     }
     va_end(argp); 
     p_i_old->next = NULL;

/* If there is onl one line ... */
     if(inline_num == 1)
     {
          line_action = ok_button_action;
          line_data = (caddr_t)new_panel;
          ok_bd = BUTTON_BD_DEF;
     }
     else
     {
          line_action = cr_action;
          ok_bd = BUTTON_BD;
     }

/* Determine width and heigh of the window */
     title_length = strlen(title);
     row_width = LINE_LENGTH + 2 + lmax;
     if(row_width < title_length)row_width = title_length;
     window_width = 2*MARGIN + theFontStruct->max_bounds.width*row_width;

     d_panel->xt = MARGIN + (row_width-title_length)*
                            theFontStruct->max_bounds.width/2 ;
     d_panel->yt = MARGIN + theFontStruct->max_bounds.ascent;
     d_panel->l_line  = LINE_LENGTH*theFontStruct->max_bounds.width;
     d_panel->x_label = MARGIN;
     d_panel->dy = theFontStruct->max_bounds.descent + Y_GAP + 1;
     inline_height = Y_OFF + Y_GAP + LINE_GAP +
                         theFontStruct->max_bounds.ascent +
                         theFontStruct->max_bounds.descent;
     window_height = d_panel->yt + 4*theFontStruct->max_bounds.descent +
                    LINE_GAP + inline_num*inline_height + 2*MARGIN +
                    theFontStruct->max_bounds.ascent +
                    2*ok_bd;

/* Position of the window */
     if(parent_w > window_width)
          window_x = (parent_w - window_width)/2;
     else
          window_x = 0;

     if(parent_h > window_height)
          window_y = (parent_h - window_height)/2;
     else
          window_y = 0;

/* Create Window */
     new_panel->xid = XCreateSimpleWindow(theDisplay, parent,
                         window_x, window_y,
                         window_width, window_height, 3,
                         bdcolor, bgcolor);

/* Save new_panel as data associated with this window id
 * and the context "xwin_context"
 */
        if(XSaveContext(theDisplay, new_panel->xid, xwin_context,
                (caddr_t) new_panel) != 0)
        {
                fprintf(stderr, "Error saving xwin_context data\n");
                exit(1);
        }
 
        XSelectInput(theDisplay, new_panel->xid, ExposureMask);
 
        XMapRaised(theDisplay, new_panel->xid);
 


     y_inline = d_panel->yt + theFontStruct->max_bounds.descent +
                LINE_GAP;

/* Fill items */
     for(p_i = d_panel->list; p_i !=NULL; p_i = p_i->next)
     {
          p_i->x_line = d_panel->x_label + p_i->x_line*
                         theFontStruct->max_bounds.width;
          p_i->y_label = y_inline + Y_OFF +
                          theFontStruct->max_bounds.ascent;
          strcpy(p_i->buff, p_i->text);
          if(inline_num !=1) line_data = (caddr_t)p_i;
          p_i->InputLine = MakeXInputLine(p_i->x_line, y_inline,
                             LINE_LENGTH, X_OFF, Y_OFF, Y_GAP, &w, &h,
                             0, bdcolor, bgcolor,
                             new_panel->xid, p_i->buff, 80,
                             line_action, line_data);
         y_inline = y_inline + inline_height;
     }

/* Make Buttons */
     y_inline = y_inline  + MARGIN;

     l_ok     = strlen(ok_text) + 2;
     l_refresh = strlen(refresh_text) + 2;
     l_cancel = strlen(cancel_text) + 1;
 
     x_butt = MARGIN + (row_width - l_cancel - l_ok -  l_refresh)*
               theFontStruct->max_bounds.width/2 - 2*BUTTON_BD - ok_bd;
     h = 3*theFontStruct->max_bounds.descent +
          theFontStruct->max_bounds.ascent;
     w = (l_ok -1)*theFontStruct->max_bounds.width;

     d_panel->bok = MakeXButton(x_butt, y_inline, w, h, ok_bd,
                    bdcolor, bgcolor, new_panel->xid, ok_text,
                    ok_button_action, (caddr_t)new_panel);

     x_butt = x_butt + 2*ok_bd +
               l_ok*theFontStruct->max_bounds.width;
     w = (l_refresh -1)*theFontStruct->max_bounds.width;

     d_panel->bref = MakeXButton(x_butt, y_inline, w, h, BUTTON_BD,
                    bdcolor, bgcolor, new_panel->xid, refresh_text,
                    refresh_button_action, (caddr_t)new_panel);

     x_butt = x_butt + 2*BUTTON_BD +
               l_refresh*theFontStruct->max_bounds.width;
     w = l_cancel*theFontStruct->max_bounds.width;

     d_panel->bcan = MakeXButton(x_butt, y_inline, w, h, BUTTON_BD,
                    bdcolor, bgcolor, new_panel->xid, cancel_text,
                    cancel_button_action, (caddr_t)new_panel);

/* Lock action */
      AppLock = 1;
      AppLockKey = new_panel->xid;

      return (new_panel);
}
/*----------------------------------------------------------------------*/
/*      I n p u t P a n e l _ h a n d l e r
 *
 *    Event handler for the InputPanel
 * 
 */
void InputPanel_handler(p_xwin)
XWIN *p_xwin;
{
        D_INPUT_PANEL   *d;
        D_ITEM          *p_i;

        d = p_xwin->data;
/* Handle events occurring in this window */
    if((theEvent.xany.window == p_xwin->xid)&&(theEvent.type == Expose)&&
        (theEvent.xexpose.count == 0))
    {
        XClearWindow(theDisplay, p_xwin->xid);
        XDrawString(theDisplay, p_xwin->xid, theGC, d->xt, d->yt,
                d->title, strlen(d->title));
        for(p_i=d->list; p_i != NULL; p_i = p_i->next)
        {
                XDrawString(theDisplay, p_xwin->xid, theGC,
                        d->x_label, p_i->y_label, p_i->label,
                        strlen(p_i->label));
                XFillRectangle(theDisplay, p_xwin->xid, theGC,
                        p_i->x_line, p_i->y_label + d->dy,
                        d->l_line, 2);
        }
     }
}

/*----------------------------------------------------------------------*/ 
/*     D e s t r o y X I n p u t P a n e l
 *
 *     Destroys an XInputPanel
 */
void  DestroyXInputPanel(p_xwin)
XWIN *p_xwin;			/* Pointer to XWIN of XInputPanel */
{
        D_INPUT_PANEL *d;
        D_ITEM *p_i, *p_n;

        d = (D_INPUT_PANEL*)p_xwin->data;
        XUnmapWindow(theDisplay, p_xwin->xid);

        for(p_i = d->list; p_i !=NULL; p_i = p_n)
        {
                p_n = p_i->next;
                DestroyXInputLine(p_i->InputLine);
                free(p_i);
        }

        DestroyXButton(d->bok);
        DestroyXButton(d->bref);
        DestroyXButton(d->bcan);
        free(d);

        XDestroyWindow(theDisplay, p_xwin->xid);
	XDeleteContext(theDisplay, p_xwin->xid, xwin_context);
        free(p_xwin);
        AppLock = 0;
        FocusedWindow = theMain;
}
/*----------------------------------------------------------------------*/
void cr_action(p_i)
D_ITEM *p_i;
{
        D_INPUT_PANEL *d;
        HideCursor(p_i->InputLine);
        if(p_i->next != NULL)
        {
                StartInputLine(p_i->next->InputLine);
        }
        else
        {
                d = (D_INPUT_PANEL*)(p_i->panel->data);
                StartInputLine(d->list->InputLine);
        }
}
/*----------------------------------------------------------------------*/
void ok_button_action(p_xwin)
XWIN *p_xwin;
{
        D_INPUT_PANEL *d;
        D_ITEM *p_i;

        d = (D_INPUT_PANEL*)(p_xwin->data);

        for(p_i = d->list; p_i!=NULL; p_i = p_i->next)
        {
                strcpy(p_i->text, p_i->buff);
        }
        DestroyXInputPanel(p_xwin);
        if(d->ok_action != NULL)
                (d->ok_action)(d->ok_data);
}
/*----------------------------------------------------------------------*/
void cancel_button_action(p_xwin)
XWIN *p_xwin;  
{
        D_INPUT_PANEL *d;

        d = (D_INPUT_PANEL*)(p_xwin->data);
        DestroyXInputPanel(p_xwin);
        if(d->cancel_action != NULL)
                (d->cancel_action)(d->cancel_data);
}
/*----------------------------------------------------------------------*/
void refresh_button_action(p_xwin)
XWIN *p_xwin;  
{
        D_INPUT_PANEL *d; 
        D_ITEM *p_i, *p_i_old;    
         
        d = (D_INPUT_PANEL*)p_xwin->data; 

        p_i_old = NULL;
 
        for(p_i = d->list; p_i!=NULL; p_i = p_i->next)
        {
                if(p_i_old != NULL)
                        HideCursor(p_i_old->InputLine);
                strcpy(p_i->buff, p_i->text);
                StartInputLine(p_i->InputLine);
                p_i_old = p_i;
        }
}
 
