/*  File: xwins.h
 *  V 2.00
 *  Defines data structures for windows
 *
 */
#ifndef XWINS_H
#define XWINS_H

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/cursorfont.h>

typedef struct APP_PARAMS
{
    char *name;
    char *cname;
    char **p_value_string;
} APP_PARAMS;

typedef struct XWIN
{
    Window   xid;
    Window   parent;
    void     *data;
    void      (*event_handler)();
} XWIN;

typedef struct
{
XWIN            *p_xwin;        /* Pointer to the XWIN struct */
unsigned        width, height;  /* Width and Height of Window */
Pixmap          pmap;           /* XID  pixmap */
unsigned        pm_w, pm_h;     /* Width and height of pixmap */
int             ox, oy;         /* position of origin */
Cursor          c1, c2;         /* Cursors to write and drag */
unsigned long   BGpix;          /* Window background pixel */
unsigned long   FGpix;          /* Window foreground pixel */
GC              gc;             /* Window graphics context */
XFontStruct     *FS;            /* Window Font Struct */
Bool            active;         /* True if it is active */
Bool            press;          /* True if button 3 is pressed */
int             dpx, dpy;       /* Pixmap displacement */
void            (*handler)();   /* Additional event handler */
void            *data;          /* Additional data */
}D_GRAPHIC_WINDOW;

typedef struct
{
XWIN           *p_xwin;            /* Pointer to the XWIN structure */
unsigned       width, height;      /* Width and Height of Window */
Pixmap         pmap;               /* Pointer to pixmap */
unsigned       pm_w, pm_h;         /* Width and height of pixmap */
int            ox, oy;             /* position of origin */
unsigned long  BGpix;              /* Window background pixel */
unsigned long  FGpix;              /* Window foreground pixel */
GC             gc;                 /* Window graphics context */
XFontStruct    *FS;                /* Window Font Struct */
Bool           active;             /* True if it is active */
unsigned long  zxmax, zymax;       /* Z factors */
unsigned long  oxmax, oymax;       /* Maximum values of origin */
XWIN           *hbar, *vbar;       /* Scroll bars */
void           (*handler)();       /* Additional event handler */
caddr_t        data;               /* Additional data */
}D_SCROLL_WINDOW;

/* Function prototypes */
void initapp(int argc, char **argv, int x, int y,
              unsigned w, unsigned h, char *app_class,
              char *icon, unsigned w_icon, unsigned h_icon,
	      char *usage_message );

void SetOptions(int argc, char** argv, int n_obl, int n_opt,
       int  n_extra, char** lname, char** sname,
       char*** pval, char* use_mess );

void GetOptions(APP_PARAMS* p_app_opt,
       int *num_opt, int* opt_start );

void SetResources(int n_extra,
        char** lname, char** sname, char*** pval );

void GetResources(APP_PARAMS* p_app_res, int* num_res );

void xwin_init(void);

void GoEventLoop( void (*Main_handler)(), void *data );
 
XWIN  *MakeXButton(int x, int y,
         unsigned width, unsigned height, unsigned bdwidth,
         unsigned long bdcolor, unsigned long bgcolor,
         Window parent, char *text, void (*button_action)(),
         caddr_t action_data);

XWIN  *MakeXMenu(int x, int y,
         unsigned bdwidth, unsigned bbdwidth,
         unsigned long bdcolor, unsigned long bgcolor,
         Window parent, int style,
         /* char *text1, void (*action1)(), caddr_t action1_args,*/
         ...  /* , NULL */);

XWIN  *MakeXWindow(int x, int y,
         unsigned width, unsigned height, unsigned bdwidth,
         unsigned long bdcolor, unsigned long bgcolor,
         Window parent, long event_mask, Cursor cursor,
         void (*handler)( XWIN * ), void *data);

XWIN  *MakeXSlide(int x, int y, unsigned bdwidth, int type,
         unsigned w, unsigned h, int bi, unsigned bw, unsigned bh, int z0, 
         int fill_type, unsigned long bdcolor, unsigned long bgcolor,
         Window parent, Bool do_it_active, void (*slide_action)(),
         caddr_t action_data);

XWIN  *MakeXInfoBox(unsigned long bdcolor, unsigned long bgcolor,
         Window parent, unsigned parent_w, unsigned parent_h,
         char **text, char *label, void (*button_action)(),
         caddr_t action_data);

XWIN  *MakeXAlertBox(unsigned long bdcolor, unsigned long bgcolor,
         Window parent, unsigned parent_w, unsigned parent_h,
         char **text, char *label1, void (*button1_action)(),
         caddr_t action_data1, char *label2, void (*button2_action)(),
         caddr_t action_data2);
 
XWIN  *MakeXQueryBox(unsigned long bdcolor, unsigned long bgcolor,
         Window parent, unsigned parent_w, unsigned parent_h,
         char **text, char *label1, void (*button1_action)(),
         caddr_t action_data1, char *label2, void (*button2_action)(),
         caddr_t action_data2, char *label3, void (*button3_action)(),
         caddr_t action_data3 );

XWIN  *MakeXInputLine(int x, int y, int nmax, int x_off, int y_off,
         int y_gap, unsigned *w, unsigned *h, unsigned bdwidth,
         unsigned long bdcolor, unsigned long bgcolor,
         Window parent, char *text, int text_size,
         void (*return_action)(),void *action_data);

XWIN  *MakeXInputPanel(unsigned long bdcolor, unsigned long bgcolor,
         Window parent, unsigned parent_w, unsigned parent_h, char *title,
         void (*ok_action)(), caddr_t ok_data,
         void (*cancel_action)(), caddr_t cancel_data,
         /*char *label1, char *text1, */.../*, NULL */);

XWIN  *MakeXScrollWindow(int x, int y, unsigned width, unsigned height,
         unsigned pm_width, unsigned pm_height,
         int ox, int oy, unsigned bdwidth,
         unsigned long bdcolor, unsigned long bgcolor, unsigned long fgcolor,
         GC gc, XFontStruct* FS, Window parent, unsigned long w_mask,
         Cursor cursor, void (* window_handler)(), caddr_t data);

XWIN  *MakeXSelector(unsigned long bdcolor, unsigned long bgcolor,
	 Window parent, unsigned parent_w, unsigned parent_h, char **title,
	 char *selection_label, void (*ok_action)(), caddr_t ok_data,
	 void (*cancel_action)(), caddr_t cancel_data,
	 char **items, int items_number,
	 char *selection);

void   MakeXFileSelector(unsigned long bdcolor, unsigned long bgcolor,
         Window parent, unsigned parent_w, unsigned parent_h, char **title,
	 char *directory, void (*ok_action)(), caddr_t ok_data,
         void (*cancel_action)(), caddr_t cancel_data,
         char *file );

int    WriteXImage(char *file, XImage *image, char *name,
         int n_colors, XColor *colors);

int    ReadXImage(char *file, XImage **p_image, char **p_name, 
         int *n_colors, XColor **p_colors,
         unsigned *w_w, unsigned *w_h, int *w_x, int *w_y,
         unsigned *w_b);

int    True2Pseudo(XImage *image_in, XImage *image_out,
           XColor *colors, int ncolors);

int    Pseudo2True(XImage *image_in, XImage **image_out, 
           XColor *colors, int ncolors , Visual *visual, int depth,
           Display *display);

int    IndeXColor(unsigned long pixel, XColor *colors, int ncolors);

void HiliteXButton(XWIN *button);

void SetXButtonStatus(XWIN *button, int status );

void DestroyXButton(XWIN *button);

XWIN *GetXWINofMenuButton(XWIN *menu, char *label);

void  PulldownXY(XWIN *menubar, int xm, int ym, unsigned bdwidth,
        unsigned bbdwidth, int *xp, int *yp);

void DestroyXMenu(XWIN *menu);

void DestroyXWindow(XWIN *xwin);

void DestroyXSlide(XWIN *slide);

int  GetXSlideZ(XWIN* slide);

void SetXSlideZ(XWIN* slide, int z);

void SetXSlideActivity(XWIN* slide, Bool act);

void DestroyXInputLine(XWIN *in_line);

void RefreshInputLine(XWIN *in_line);

void StartInputLine(XWIN *in_line);

void HideCursor(XWIN *in_line);

void DestroyXScrollWindow(XWIN *scrwin);

void ClearXScrollWindowPixmap(XWIN *scrwin);

Pixmap GetPixmapOfXScrollWindow(XWIN *scrwin, unsigned *pm_w, unsigned *pm_h);

void GetXScrollWindowValues(XWIN *scrwin, unsigned *width, unsigned *height,
       int *ox, int *oy);

void RefreshXScrollWindow(XWIN *scrwin);

void SetXScrollWindowBG(XWIN *scrwin, unsigned long pixel);

unsigned long GetXScrollWindowBG(XWIN *scrwin);

void SetXScrollWindowFG(XWIN *scrwin, unsigned long pixel);

unsigned long GetXScrollWindowFG(XWIN *scrwin);

void AddNewEvent(Window xid, long event_mask);

void IgnoreEvent(Window xid, long event_mask);

void SizeOfString(char*str, unsigned *w, unsigned *h, XFontStruct *FS);

void SizeOfText(char* text[], unsigned *w, unsigned  *h, XFontStruct *FS);

void DrawText(Window w, int x, int y, char* text[]);

unsigned long swaplong(unsigned long *l);

unsigned short swapshort(unsigned short *s);

unsigned char swapbits(char *s);

int system_byte_order(void);

void SetXTimer(long sec, long microsec, void (*handler)(),
               caddr_t data);

void InitXTimer(void);

int read_cms(char *file, int *size, int *initial, XColor *colors);

int write_cms(char *file, int size, int initial, XColor *colors); 

unsigned long two_to(int n);

void bitsif(unsigned long mask, int *initial_bit, int *final_bit);

void SetStandardXColor(XColor *colors, int *ncolors);

void SetPixelXColor(XColor *colors, int size, int initial,
			int offset, Visual *visual);

int GetXColor(unsigned red, unsigned green, unsigned blue,
		 XColor *colors, int n_colors);

void CreateXAuxPmap();
void DestroyXAuxPmap();
void DrawXStringAngle(Drawable d, int x, int y,
			 char *string, float angle, GC gc);
void ClearXPixmap(Pixmap pm, unsigned w, unsigned h);

XWIN  *MakeXGraphicWindow(int x, int y, unsigned width, unsigned height,
         unsigned pm_width, unsigned pm_height, int ox, int oy,
         unsigned bdwidth, unsigned long bdpix,
         unsigned long bgpix, unsigned long fgpix, GC gc, XFontStruct *FS,
         Window parent, Cursor cursor,
         void (* window_handler)(), caddr_t data);

void DestroyXGraphicWindow(XWIN* p_xwin);
void RefreshXGraphicWindow(XWIN* p_xwin);
void ClearXGraphicWindowPixmap(XWIN* p_xwin);

/* XMenu Styles */
#define MENUBAR_STYLE  0
#define PULLDOWN_STYLE 1

/* XButton status */
#define OFF    0
#define NORMAL 1
#define CROSS_HILITE 2
#define HILITE 3

/* XSlide types */
#define HORIZONTAL  0
#define UP          1
#define DOWN        2

/* XSlide fill_types */
#define BOTH_SIDES 0
#define ONE_SIDE 1

/* Size of Auxiliary pixmap */
#define WIDTH_AUXPIXMAP 1000
#define HEIGHT_AUXPIXMAP 100

/* Global variable */
#ifdef XWIN_UTIL

XContext      xwin_context;

#else

extern XContext       xwin_context;

#endif /* #ifdef XWIN_UTIL */

#endif /* #ifndef XWINS_H */
